/* mainProgram.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"
#include "mainProgram.h"

#ifdef HAVE_GETOPT_LONG_ONLY
#include <getopt.h>
#else
#include "gnu_getopt.h"
#endif

MainProgram::
MainProgram(const string &program_name) {
  _program_name = program_name;

  define_opt("h", false);
}

void MainProgram::
define_opt(const string &opt, bool takes_arg) {
  Opt optdef;
  optdef._opt = opt;
  optdef._takes_arg = takes_arg;
  _opts.push_back(optdef);
}

void MainProgram::
command_line(int argc, char *argv[]) {
  // Build up the long options list and the short options string for
  // getopt_long().
  vector<struct option> long_options;
  string short_options;

  int i;
  for (i = 0; i < _opts.size(); i++) {
    if (_opts[i]._opt.length() == 1) {
      short_options += _opts[i]._opt;
      if (_opts[i]._takes_arg) {
	short_options += ':';
      }

    } else {
      struct option opt;
      opt.name = _opts[i]._opt.c_str();
      opt.has_arg = (_opts[i]._takes_arg) ? required_argument : no_argument;
      opt.flag = NULL;

      // Return an index into the _opts array, offset by 256 so we
      // don't confuse it with any of the short opts.
      opt.val = i + 256;

      long_options.push_back(opt);
    }
  }

  struct option opt;
  memset(&opt, 0, sizeof(opt));
  long_options.push_back(opt);

  extern char *optarg;
  const char *short_opts = short_options.c_str();
  const struct option *long_opts = &long_options[0];

  int flag = getopt_long_only(argc, argv, short_opts, long_opts, NULL);
  while (flag != EOF) {
    string arg;
    if (optarg != NULL) {
      arg = optarg;
    }

    if (flag < 256) {
      char ch = (char)flag;
      handle_opts(string(&ch, 1), arg);
    } else {
      handle_opts(_opts[flag - 256]._opt, arg);
    }

    flag = getopt_long_only(argc, argv, short_opts, long_opts, NULL);
  }

  Args remaining_args;
  
  for (i = optind; i < argc; i++) {
    remaining_args.push_back(argv[i]);
  }

  handle_args(remaining_args);
}

void MainProgram::
init() {
}

void MainProgram::
run() {
}

void MainProgram::
handle_opts(const string &opt, const string &arg) {
  if (opt == "h") {
    help();
    exit(1);

  } else if (opt == "?") {    
    usage();
    exit(1);
    
  } else {
    cerr << "Internal error! Unhandled switch: -" << opt << "\n";
    abort();
  }
}

void MainProgram::
handle_args(const Args &args) {
  if (!args.empty()) {
    usage();
    exit(1);
  }
}

void MainProgram::
help() {
  // Normally, the deriving program will put a paragraph here
  // introducing the program and describing its function.
  usage();
}

void MainProgram::
usage() {
  cerr << "\nUsage:\n\n";

  if (_opts.empty()) {
    cerr << _program_name;
    describe_args();

  } else {
    cerr << _program_name << " [opts]";
    describe_args();
    cerr << "Options:\n\n";
    describe_options();
    cerr << "\n";
  }
}

void MainProgram::
describe_options() {
}

void MainProgram::
describe_args() {
  cerr << "\n\n";
}
