/* importListDialog.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef IMPORTLISTDIALOG_H
#define IMPORTLISTDIALOG_H

#include "palomaDialog.h"
#include "importList.h"
#include "volume.h"

#include <gtk--.h>
#include <sys/types.h>

class ImportDialog;
class Volume;

class ImportListDialog : public PalomaDialog {
public:
  enum Disposition {
    D_leave,
    D_copy,
    D_convert
  };

  // These classes are used to collect the songs into CD's with
  // associated artists prior to actually importing.
  class Artist {
  public:
    Artist(const string &name);
    int operator < (const Artist &other) const;

    string _name;
    int _artist_id;
  };

  class CD {
  public:
    CD(const string &title, const Artist *performer, const Artist *writer);
    int operator < (const CD &other) const;

    string _title;
    const Artist *_performer;
    const Artist *_writer;
    int _cd_id;
  };

  class Song {
  public:
    Song();

    const Artist *_performer;
    const Artist *_writer;
    string _relpath;
    int _type;
    string _name;
    const CD *_cd;
    int _track_num;
    TableDataProxy _db_song;
  };

  typedef set<Artist> Artists;
  typedef set<CD> CDs;
  typedef vector<Song *> Songs;

  // The ImportListDialog becomes the owner of the good_files and
  // bad_files lists, and will free them when it destructs.
  ImportListDialog(const Dataset &data, ImportDialog *id,
		   const Volume &volume, Disposition disp,
		   int convert_to_type, const string &root_prefix,
		   ImportList *good_files, ImportList *bad_files);
  ~ImportListDialog();

private:
  void got_column_click(gint colnum);
  void picked_popup_clean_spaces();
  void picked_popup_fix_capitalization();
  void picked_popup_type(ImportList::ColumnType type);
  void ok_button_clicked();
  void cancel_button_clicked();
  void import_dialog_deleted();

  void layout_window();

  Gtk_VPaned _paned;
  Gtk_VBox _good_box;
  Gtk_Label _good_label;
  Gtk_ScrolledWindow _good_window;
  Gtk_CList *_good_clist;
  Gtk_VBox _bad_box;
  Gtk_Label _bad_label;
  Gtk_ScrolledWindow _bad_window;
  Gtk_CList *_bad_clist;
  Gtk_CheckButton _playlist_check;
  Gtk_Button _ok_button;
  Gtk_Button _cancel_button;
  Gtk_Menu *_popup_menu;


  const Artist *define_artist(const string &name);
  const CD *define_cd(const string &title, const string &performer,
		      const string &writer);
  void collect_songs();
  void import_songs();
  void build_playlist();

  Volume _volume;
  Disposition _disp;
  string _volume_prefix;
  int _convert_to_type;
  string _root_prefix;
  ImportList *_good_files;
  ImportList *_bad_files;
  ImportDialog *_import_dialog;

  int _popup_colnum;


  Artists _artists;
  CDs _cds;
  Songs _songs;
};

#endif


